@@ -141,6 +141,7 @@ class Experiment(BaseExperiment):
         display_summary=None,
         log_env_cpu=True,
         display_summary_level=1,
+        optimizer_data=None,
     ):
         """
         Creates a new experiment on the Comet.ml frontend.
@@ -214,6 +215,7 @@ class Experiment(BaseExperiment):
             log_env_host=log_env_host,
             display_summary_level=display_summary_level,
             log_env_cpu=log_env_cpu,
+            optimizer_data=optimizer_data,
         )
 
         self.ws_connection = None
@@ -294,6 +296,7 @@ class Experiment(BaseExperiment):
             self.is_github,
             self.focus_link,
             self.upload_limit,
+            self.asset_upload_limit,
             feature_toggles,
             initial_offset,
             self.upload_web_asset_url_prefix,
@@ -383,15 +386,17 @@ class Experiment(BaseExperiment):
 
         # Display throttling message
         try:
-            experiment_metadata = self.rest_api_client.get_experiment_metadata(self.id)
-
-            if experiment_metadata.get("throttle", False):
+            if self._check_experiment_throttled():
                 LOGGER.warning(ONLINE_EXPERIMENT_THROTTLED)
         except Exception:
             LOGGER.debug("Failed to check experiment metadata", exc_info=True)
 
         return successful_clean
 
+    def _check_experiment_throttled(self):
+        experiment_metadata = self.rest_api_client.get_experiment_metadata(self.id)
+        return experiment_metadata.get("throttle", False)
+
     @property
     def url(self):
         """
@@ -828,6 +833,7 @@ class ExistingExperiment(Experiment):
                 self.is_github,
                 self.focus_link,
                 self.upload_limit,
+                self.asset_upload_limit,
                 feature_toggles,
                 initial_offset,
                 self.upload_web_asset_url_prefix,
