@@ -18,6 +18,7 @@ import six
 
 from .._typing import Any, Optional
 from ..experiment import BaseExperiment
+from ..predictor import Predictor
 from ..utils import get_time_monotonic
 
 LOGGER = logging.getLogger(__name__)
@@ -86,6 +87,7 @@ def build_base_callback(base):
 def build_predictive_early_stopping_keras_callback(base):
     class PredictiveEarlyStoppingKerasCallback(base):  # type: ignore
         def __init__(self, predictor, predict_epoch_rate=1):
+            # type: (Predictor, int) -> None
             super(PredictiveEarlyStoppingKerasCallback, self).__init__()
             self.predictor = predictor
             self.predict_epoch_rate = predict_epoch_rate
@@ -138,57 +140,93 @@ def build_keras_callback(base):
             self.our_step = 0
 
         def on_epoch_begin(self, epoch, logs=None):
-            # This function should only be called during train mode.
-            LOGGER.debug("On epoch begin %s %s", epoch, logs)
-            self.experiment.set_epoch(epoch)
-            self.epoch_start_time = get_time_monotonic()
+            try:
+                # This function should only be called during train mode.
+                LOGGER.debug("On epoch begin %s %s", epoch, logs)
+                self.experiment.set_epoch(epoch)
+                self.epoch_start_time = get_time_monotonic()
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_epoch_begin; ignoring",
+                    exc_info=True,
+                )
 
         def on_epoch_end(self, epoch, logs=None):
-            # This function should only be called during train mode.
-            LOGGER.debug("On epoch end %s %s", epoch, logs)
-            if self.log_metrics:
-                if self.epoch_start_time is not None:
-                    self.experiment._log_metric(
-                        "epoch_duration",
-                        get_time_monotonic() - self.epoch_start_time,
-                        step=self.our_step,
-                        framework="keras",
-                    )
-                    self.epoch_start_time = None
-                self.experiment.log_epoch_end(epoch, step=self.our_step)
-                if logs:
-                    for name, val in logs.items():
+            try:
+                # This function should only be called during train mode.
+                LOGGER.debug("On epoch end %s %s", epoch, logs)
+                if self.log_metrics:
+                    if self.epoch_start_time is not None:
                         self.experiment._log_metric(
-                            name, val, step=self.our_step, framework="keras"
+                            "epoch_duration",
+                            get_time_monotonic() - self.epoch_start_time,
+                            step=self.our_step,
+                            framework="keras",
                         )
+                        self.epoch_start_time = None
+                    self.experiment.log_epoch_end(epoch, step=self.our_step)
+                    if logs:
+                        for name, val in logs.items():
+                            self.experiment._log_metric(
+                                name, val, step=self.our_step, framework="keras"
+                            )
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_epoch_end; ignoring",
+                    exc_info=True,
+                )
 
         def on_batch_begin(self, batch, logs=None):
-            # This function called directly when in train mode.
-            LOGGER.debug("On batch begin %s %s", batch, logs)
+            try:
+                # This function called directly when in train mode.
+                LOGGER.debug("On batch begin %s %s", batch, logs)
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_batch_begin; ignoring",
+                    exc_info=True,
+                )
 
         def on_batch_end(self, batch, logs=None):
             """
             Logs training metrics.
             """
-            # This function called directly when in train mode.
-            LOGGER.debug("On batch end %s %s", batch, logs)
+            try:
+                # This function called directly when in train mode.
+                LOGGER.debug("On batch end %s %s", batch, logs)
 
-            self.our_step += 1
+                self.our_step += 1
 
-            if self.experiment.batch_report_rate > 0:
-                if batch is not None and isinstance(batch, six.integer_types):
-                    if batch % self.experiment.batch_report_rate == 0:
+                if self.experiment.batch_report_rate > 0:
+                    if batch is not None and isinstance(batch, six.integer_types):
+                        if batch % self.experiment.batch_report_rate == 0:
+                            self._send_batch_messages(logs)
+                    else:
                         self._send_batch_messages(logs)
-                else:
-                    self._send_batch_messages(logs)
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_batch_end; ignoring",
+                    exc_info=True,
+                )
 
         def on_train_batch_end(self, batch, logs=None):
-            # No context added here, to match previous behavior:
-            self.on_batch_end(batch, logs)
+            try:
+                # No context added here, to match previous behavior:
+                self.on_batch_end(batch, logs)
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_train_batch_end; ignoring",
+                    exc_info=True,
+                )
 
         def on_test_batch_end(self, batch, logs=None):
-            with self.experiment.validate():
-                self.on_batch_end(batch, logs)
+            try:
+                with self.experiment.validate():
+                    self.on_batch_end(batch, logs)
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_test_batch_end; ignoring",
+                    exc_info=True,
+                )
 
         def _send_batch_messages(self, logs):
             if logs and self.log_metrics:
@@ -201,45 +239,61 @@ def build_keras_callback(base):
             """
             Sets model graph.
             """
-            LOGGER.debug("On train begin %s", logs)
+            try:
+                LOGGER.debug("On train begin %s", logs)
 
-            if self.log_graph:
-                model_graph = get_keras_json_model(self.experiment, self.model)
+                if self.log_graph:
+                    model_graph = get_keras_json_model(self.experiment, self.model)
 
-                if model_graph:
-                    self.experiment._set_model_graph(model_graph, framework="keras")
-                else:
-                    LOGGER.debug("Empty graph model, skipping")
-
-            self.experiment._log_other(
-                "trainable_params", self.model.count_params(), framework="keras"
-            )
-
-            if self.log_params:
-                if logs:
-                    for k, v in logs.items():
-                        self.experiment._log_parameter(k, v, framework="keras")
-
-                # Keras Callback doesn't set this parameter at creation by default
-                if hasattr(self, "params") and self.params:
-                    for k, v in self.params.items():
-                        if k != "metrics":
-                            self.experiment._log_parameter(k, v, framework="keras")
+                    if model_graph:
+                        self.experiment._set_model_graph(model_graph, framework="keras")
+                    else:
+                        LOGGER.debug("Empty graph model, skipping")
 
                 try:
-                    optimizer_name = self.model.optimizer.__class__.__name__
-                    config = self.model.optimizer.get_config()
-                    for key, value in config.items():
-                        self.experiment._log_parameter(
-                            optimizer_name + "_" + key, value, framework="keras"
-                        )
-                except Exception:
-                    LOGGER.debug(
-                        "Failed to extract optimizer information", exc_info=True
+                    trainable_params = self.model.count_params()
+                    self.experiment._log_other(
+                        "trainable_params", trainable_params, framework="keras"
                     )
+                except Exception:
+                    LOGGER.debug("Failed to count params in model", exc_info=True)
+
+                if self.log_params:
+                    if logs:
+                        for k, v in logs.items():
+                            self.experiment._log_parameter(k, v, framework="keras")
+
+                    # Keras Callback doesn't set this parameter at creation by default
+                    if hasattr(self, "params") and self.params:
+                        for k, v in self.params.items():
+                            if k != "metrics":
+                                self.experiment._log_parameter(k, v, framework="keras")
+
+                    try:
+                        optimizer_name = self.model.optimizer.__class__.__name__
+                        config = self.model.optimizer.get_config()
+                        for key, value in config.items():
+                            self.experiment._log_parameter(
+                                optimizer_name + "_" + key, value, framework="keras"
+                            )
+                    except Exception:
+                        LOGGER.debug(
+                            "Failed to extract optimizer information", exc_info=True
+                        )
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_train_begin; ignoring",
+                    exc_info=True,
+                )
 
         def on_train_end(self, *args, **kwargs):
-            LOGGER.debug("On train end %r", locals())
+            try:
+                LOGGER.debug("On train end %r", locals())
+            except Exception:
+                LOGGER.warning(
+                    "An unknown exception happened in Keras callback on_train_end; ignoring",
+                    exc_info=True,
+                )
 
     return KerasCallback
 
