@@ -35,7 +35,8 @@ def fit_logger(keras_type, framework, experiment, original, *args, **kwargs):
         try:
             callback = experiment.get_callback(keras_type)
         except Exception:
-            LOGGER.warning(GET_CALLBACK_FAILURE, framework)
+            LOGGER.warning(GET_CALLBACK_FAILURE, framework, exc_info=True)
+            return
 
         if "callbacks" in kwargs and kwargs["callbacks"] is not None:
             callbacks = kwargs["callbacks"]
