@@ -210,6 +210,11 @@ class OfflineExperiment(BaseExperiment):
         """
         pass
 
+    def display_project(self, *args, **kwargs):
+        """ Do nothing
+        """
+        pass
+
     def _start(self):
         self.start_time = local_timestamp()
         super(OfflineExperiment, self)._start()
@@ -452,7 +457,8 @@ class OfflineSender(object):
             self.project_id,
             self.is_github,
             self.focus_link,
-            self.upload_limit,
+            _,
+            _,
             feature_toggles,
             initial_offset,
             upload_web_asset_url_prefix,
