@@ -18,9 +18,11 @@ import logging
 import os
 
 from ._logging import OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED
+from ._typing import Any, Dict, Optional
 from .config import get_api_key, get_config
 from .connection import get_optimizer_api
 from .exceptions import InvalidOptimizerParameters, OptimizerException
+from .experiment import BaseExperiment
 from .predictor import Predictor
 
 LOGGER = logging.getLogger(__name__)
@@ -198,7 +200,30 @@ class Optimizer(object):
             else:
                 break
 
+    def get_parameters(self):
+        """
+        `Optimizer.get_parameters()` will iterate over all possible parameters
+        for this sweep or search. All parameters combinations will be emitted
+        once (unless performing multiple trials per parameter set).
+
+        Example:
+
+            ```python
+            >>> for parameters in optimizer.get_parameters():
+            ...     experiment = comet_ml.Experiment()
+            ...     loss = fit(x, y)
+            ...     experiment.log_metric("loss", loss)
+            ```
+        """
+        while True:
+            data = self.next_data()
+            if data is not None:
+                yield data
+            else:
+                break
+
     def next(self, **kwargs):
+        # type: (Dict[str, str]) -> Optional[BaseExperiment]
         """
         `Optimizer.next()` will return the next experiment for this
         sweep or search. All experiments will have a unique set of
@@ -221,8 +246,8 @@ class Optimizer(object):
             ...     experiment.log_metric("loss", loss)
             ```
         """
-        data = self._api.optimizer_next(self.id)
-        if data is not None:
+        data = self.next_data()
+        if data:
             return self._create_experiment(
                 data["parameters"],
                 data["pid"],
@@ -230,6 +255,30 @@ class Optimizer(object):
                 data["count"],
                 experiment_kwargs=kwargs,
             )
+
+        return None
+
+    def next_data(self):
+        # type: () -> Optional[Dict[str, Any]]
+        """
+        `Optimizer.next_data()` will return the next parameters in the Optimizer
+        sweep.
+
+        Normally, you would not call this directly, but use the generator
+        `Optimizer.get_parameters()`
+
+        Example:
+
+            ```python
+            >>> experiment = optimizer.next_data()
+            ```
+        """
+        data = self._api.optimizer_next(self.id)
+        if data is not None:
+            # Include the optimizer ID so Experiment can recreate the Optimizer
+            # object if needed
+            data["id"] = self.id
+            return data
         else:
             # Check the optimizer status
             instance = self._api.optimizer_status(self.id)
@@ -532,6 +581,7 @@ class Optimizer(object):
         return v1 != v2
 
     def _create_experiment(self, parameters, pid, trial, count, experiment_kwargs):
+        # type: (Any, Any, Any, Any, Dict[str, Any]) -> BaseExperiment
         """
         Instantiates an Experiment, OfflineExperiment, or
         callable.
@@ -551,16 +601,18 @@ class Optimizer(object):
             experiment_kwargs["api_key"] = self.api_key
 
         if self.experiment_class == "Experiment":
-            exp = Experiment(**experiment_kwargs)
+            exp = Experiment(**experiment_kwargs)  # type: BaseExperiment
         elif self.experiment_class == "OfflineExperiment":
-            exp = OfflineExperiment(**experiment_kwargs)
+            exp = OfflineExperiment(**experiment_kwargs)  # type: BaseExperiment
         elif callable(self.experiment_class):
             exp = self.experiment_class(**experiment_kwargs)
         else:
             raise OptimizerException(
                 "Invalid experiment_class: %s" % self.experiment_class
             )
+
         exp._set_optimizer(self, pid, trial, count)
+
         if self.predictor_spec:
             LOGGER.debug("Setting predictor in experiment: %s", self.predictor_spec)
             if isinstance(exp, OfflineExperiment):
