@@ -15,7 +15,7 @@
 import logging
 import time
 
-from ._typing import Any, Dict, List, Optional, Tuple
+from ._typing import Any, Dict, List, Optional, Tuple, Union
 from .config import get_config
 from .connection import (
     LowLevelHTTPClient,
@@ -75,6 +75,9 @@ class Predictor(object):
         if not 0.0 < self.threshold <= 1.0:
             raise ValueError("Threshold must be set between 0.0 and 1.0")
 
+        if not self.patience > 0:
+            raise ValueError("Patience must to be set to greater than 0")
+
         # The minimum number of data points from a loss curve
         # required to make a prediction
         self.min_samples = 10
@@ -127,15 +130,16 @@ class Predictor(object):
 
         if mode is None:
             self._set_mode(optimizer_id)
-
         elif mode == "global":
             self._setup_global_mode(optimizer_id)
-            self.api = api if api else get_optimizer_api_client(self.experiment.api_key)
-
         elif mode == "local":
             LOGGER.warning("Predictor Local Mode is still experimental.")
             self.mode = "local"
 
+        # We need an API client in global mode
+        if self.mode == "global":
+            self.api = api if api else get_optimizer_api_client(self.experiment.api_key)
+
     def _set_mode(self, optimizer_id=None):
         """ Try to use the global mode but fallback on the local mode if not optimizer_id could be found. Show a WARNING in that case.
         """
@@ -223,6 +227,22 @@ class Predictor(object):
     def _elapsed_time(self):
         return time.time() - self.last_prediction_timestamp
 
+    def _set_and_log_stopping_metrics(self):
+        self.stop_step = self.step
+        self.stop_val = self.losses[-1]
+        self.stop_min = min(self.losses)
+
+        self.experiment.log_other("predictor_stop_step", self.stop_step)
+        self.experiment.log_metric("predictor_wait", self.wait)
+        msg = (
+            "Predicted probability of improvement failed"
+            " to exceed threshold {} for {} evaluations".format(
+                self.threshold, self.patience
+            )
+        )
+        self.stop_reason = msg
+        self.experiment.log_other("predictor_stop_reason", msg)
+
     def _local_stop_early(self, **data):
         # type: (Any) -> bool
         current_loss = self.losses[-1]
@@ -250,36 +270,21 @@ class Predictor(object):
                 "predictor_p_improvement": p_improvement,
                 "predictor_threshold": self.threshold,
                 "predictor_patience": self.patience,
-                "predictor_wait": self.wait,
             },
             step=self.step,
         )
 
-        # If the loss is improving, reset the wait count
-        # Every time we see an improvement, run the callback
-        if current_loss < lowest_min:
-            self.wait = 0
-
         if p_improvement <= self.threshold:
             self.wait += 1
 
+        # If the loss is improving, reset the wait count
+        if current_loss < lowest_min:
+            self.wait = 0
+
         # If patience is exceeded, stop training
         if self.wait >= self.patience:
             if self.stop_step is None:
-                self.stop_step = self.step
-                self.stop_val = self.losses[-1]
-                self.stop_min = min(self.losses)
-
-            self.experiment.log_other("predictor_stop_step", self.stop_step)
-            msg = (
-                "Predicted probability of improvement failed"
-                " to exceed threshold {} for {} evaluations".format(
-                    self.threshold, self.patience
-                )
-            )
-            self.stop_reason = msg
-            self.experiment.log_other("predictor_stop_reason", msg)
-
+                self._set_and_log_stopping_metrics()
             if self.best_callback:
                 try:
                     self.best_callback(self, current_best)
@@ -302,8 +307,14 @@ class Predictor(object):
             return False
 
         experiment_count = state.get("experiment_count")
+        best_metric_experiment_key = state.get("best_metric_experiment_key")
         best_metric = state.get("best_metric")
 
+        # If the current experiment has the best metric
+        # Do not stop
+        if best_metric_experiment_key == self.experiment.id:
+            return False
+
         lower = state.get("lower")
         mean = state.get("mean")
         upper = state.get("upper")
@@ -312,13 +323,13 @@ class Predictor(object):
         self.experiment.log_metrics(
             {
                 "predictor_mean": mean,
-                "predictor_max": upper,
-                "predictor_min": lower,
+                "predictor_upper": upper,
+                "predictor_lower": lower,
+                "predictor_p_improvement": p_improvement,
                 "predictor_threshold": self.threshold,
                 "predictor_patience": self.patience,
-                "predictor_wait": self.wait,
-                "predictor_p_improvement": p_improvement,
-            }
+            },
+            step=self.step,
         )
 
         if experiment_count > self.n_init_experiments:
@@ -327,36 +338,23 @@ class Predictor(object):
 
         if self.wait >= self.patience:
             if self.stop_step is None:
-                self.stop_step = self.step
-                self.stop_val = self.losses[-1]
-                self.stop_min = min(self.losses)
-
-                self.experiment.log_other("predictor_stop_step", self.stop_step)
-                msg = (
-                    "Predicted probability of improvement failed"
-                    " to exceed threshold {} for {} evaluations".format(
-                        self.threshold, self.patience
+                self._set_and_log_stopping_metrics()
+            if self.best_callback:
+                try:
+                    self.best_callback(self, best_metric)
+                except Exception:
+                    LOGGER.error(
+                        "Error calling best callback with best value: %r",
+                        best_metric,
+                        exc_info=True,
                     )
-                )
-                self.experiment.log_other("predictor_stop_reason", msg)
-                self.stop_reason = msg
-                if self.best_callback:
-                    try:
-                        self.best_callback(self, best_metric)
 
-                    except Exception:
-                        LOGGER.error(
-                            "Error calling best callback with best value: %r",
-                            best_metric,
-                            exc_info=True,
-                        )
-
-                return True
+            return True
 
         return False
 
     def _get_trial_metrics(self):
-        # type: () -> Optional[Tuple[Optional[int], Optional[float]]]
+        # type: () -> Optional[Tuple[Optional[int], Optional[str], Optional[float]]]
         count = 0
         while count <= self.MAX_TRIES:
             try:
@@ -383,12 +381,13 @@ class Predictor(object):
             break
 
         experiment_count = response_data.get("experimentCount")
+        experiment_key = response_data.get("experimentKey")
         best_metric = response_data.get("metricValue")
 
-        return experiment_count, best_metric
+        return experiment_count, experiment_key, best_metric
 
     def _get_trial_state(self, data):
-        # type: (Dict[str, Any]) -> Optional[Dict[str, float]]
+        # type: (Dict[str, Any]) -> Optional[Dict[str, Union[str, float]]]
         """
         Internal method only used in global mode to retrieve best metric
         in an optimizer run
@@ -400,7 +399,7 @@ class Predictor(object):
         if trial_metrics is None:
             return None
 
-        experiment_count, best_metric = trial_metrics
+        experiment_count, experiment_key, best_metric = trial_metrics
         if (experiment_count is None) or (best_metric is None):
             return None
 
@@ -417,6 +416,7 @@ class Predictor(object):
             "upper": upper,
             "prob_improvement": p_improvement,
             "experiment_count": experiment_count,
+            "best_metric_experiment_key": experiment_key,
             "best_metric": best_metric,
         }
 
@@ -429,7 +429,7 @@ class Predictor(object):
         # produce unwanted behaviour
 
         try:
-            # Check for cases where model is not stopped after _global_stop_early returns True
+            # Check for cases where model is not stopped after stop_early returns True
             if self.wait >= self.patience:
                 LOGGER.debug(
                     "Patience Exceeded: %s >= %s; stopping", self.wait, self.patience
@@ -542,7 +542,7 @@ class Predictor(object):
             trial_metrics = self._get_trial_metrics()
 
             if trial_metrics is not None:
-                _, best_metric = trial_metrics
+                _, _, best_metric = trial_metrics
                 msg["best-sweep-value"] = best_metric
 
             return msg
