@@ -327,11 +327,11 @@ def array_to_image_fp(
 
 def array_to_image(
     array,
-    image_scale,
-    image_shape,
-    image_colormap,
-    image_minmax,
-    image_channels,
+    image_scale=1.0,
+    image_shape=None,
+    image_colormap=None,
+    image_minmax=None,
+    image_channels=None,
     mode=None,
 ):
     # type: (Any, float, Optional[Sequence[int]], Optional[str], Optional[Sequence[float]], Optional[str], Optional[str]) -> Optional[Any]
@@ -597,7 +597,7 @@ def write_file_like_to_tmp_file(file_like_object, tmpdir):
 
 
 def table_to_fp(tabular_data, delim, headers):
-    # type: (Any, str, Union(Sequence[str], bool)) -> IO
+    # type: (Any, str, Union[Sequence[str], bool]) -> IO
     if isinstance(headers, bool):
         add_headers = headers
     else:
@@ -1964,6 +1964,20 @@ def convert_to_list(items, dtype=None):
         return [convert_to_scalar(item, dtype=dtype) for item in items]
 
 
+def convert_to_string(user_data):
+    # type: (Any) -> str
+    """
+    Given an object, return it as a string.
+    """
+    if hasattr(user_data, "numpy"):
+        user_data = user_data.numpy()
+
+    if isinstance(user_data, bytes) and not isinstance(user_data, str):
+        user_data = user_data.decode("utf-8")
+
+    return str(user_data)
+
+
 def convert_to_scalar(user_data, dtype=None):
     # type: (Any, Optional[Type]) -> Any
     """
